@@ -25,35 +25,49 @@
 #include <config.h>
 #include <gnome.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "gtcd_public.h"
 #include "keybindings.h"
 #include "prefs.h"
+#include "cddb_props.h"
 
-static GtkWidget *pref_window=NULL;
+static GtkWidget *pref_window;
 
+/* private functions */
+static void color_set_cb(GnomeColorPicker *cp, guint pr, guint pg, guint pb);
+static void start_toggle_cb(GtkWidget *widget, gpointer data);
+static void check_changed_cb(GtkWidget *widget, gboolean *data);
+static GtkWidget *create_start_frame(void);
+static void exit_toggle_cb(GtkWidget *widget, gpointer data);
+static GtkWidget *create_exit_frame(void);
+static void dev_entry_changed_cb(GtkWidget *widget, gpointer data);
+static void font_ok_clicked_cb(GtkWidget *widget, GtkWidget *fs);
+static void font_cancel_clicked_cb(GtkWidget *widget, GtkWidget *fs);
+static void font_button_cb(GtkWidget *widget, gpointer *data);
+static GtkWidget *create_general_frame(void);
+static GtkWidget *create_page(void);
+static void apply_cb(GtkWidget *widget, void *data);
+static GtkWidget *key_page(void);
+static int entry_changed(GtkWidget *widget, GdkEvent *ev, KeyBinding *kb);
+
+/* public functions */
 void load_prefs(tcd_prefs *prop);
 void save_prefs(tcd_prefs *prop);
 void changed_cb(GtkWidget *widget, void *data);
-void color_set_cb(GnomeColorPicker *cp, int pr, int pg, int pb);
-void start_toggle_cb(GtkWidget *widget, gpointer data);
-void check_changed_cb(GtkWidget *widget, gboolean *data);
-GtkWidget *create_start_frame(void);
-void exit_toggle_cb(GtkWidget *widget, gpointer data);
-GtkWidget *create_exit_frame(void);
-void dev_entry_changed_cb(GtkWidget *widget, gpointer data);
-void font_ok_clicked_cb(GtkWidget *widget, GtkWidget *fs);
-void font_cancel_clicked_cb(GtkWidget *widget, GtkWidget *fs);
-void font_button_cb(GtkWidget *widget, gpointer *data);
-GtkWidget *create_general_frame(void);
-GtkWidget *create_page(void);
-void apply_cb(GtkWidget *widget, void *data);
 void preferences(GtkWidget *widget, void *data);
-GtkWidget *key_page(void);
 
 void load_prefs(tcd_prefs *prop)
 {
+#if defined(sun) || defined(__sun__)
+#if defined(SVR4) || defined(__svr4__)
+    prop->cddev=gnome_config_get_string    ("/gtcd/cdrom/device=/vol/dev/aliases/cdrom0");
+#else
+    prop->cddev=gnome_config_get_string    ("/gtcd/cdrom/device=/dev/rcd0");
+#endif
+#else
     prop->cddev=gnome_config_get_string    ("/gtcd/cdrom/device=/dev/cdrom");
+#endif
     prop->handle=gnome_config_get_bool     ("/gtcd/ui/handle=false");
     prop->tooltip=gnome_config_get_bool    ("/gtcd/ui/tooltip=true");
     prop->time_display=gnome_config_get_int("/gtcd/ui/time_display=0");
@@ -67,6 +81,17 @@ void load_prefs(tcd_prefs *prop)
     prop->start_action=gnome_config_get_int        ("/gtcd/general/start_action=0");
     prop->close_tray_on_start=gnome_config_get_bool("/gtcd/general/close_tray_on_start=false");
 
+    prop->quit.key =    gnome_config_get_int("/gtcd/keybindings/quit=81");
+    prop->play.key =    gnome_config_get_int("/gtcd/keybindings/play=80");
+    prop->stop.key =    gnome_config_get_int("/gtcd/keybindings/stop=83");
+    prop->tracked.key = gnome_config_get_int("/gtcd/keybindings/tracked=84");
+    prop->eject.key =   gnome_config_get_int("/gtcd/keybindings/eject=69");
+    prop->back.key =    gnome_config_get_int("/gtcd/keybindings/back=45");
+    prop->forward.key = gnome_config_get_int("/gtcd/keybindings/forward=43");
+
+/* cddb stuff, used by cddbslave. */
+    prop->cddb_server = gnome_config_get_string("/cddbslave/server/address=us.cddb.com");
+    prop->cddb_port   = gnome_config_get_int("/cddbslave/server/port=8880");
 }
 
 void save_prefs(tcd_prefs *prop)
@@ -84,32 +109,42 @@ void save_prefs(tcd_prefs *prop)
     gnome_config_set_int ("/gtcd/general/exit_action", prop->exit_action);
     gnome_config_set_int ("/gtcd/general/start_action", prop->start_action);
     gnome_config_set_bool("/gtcd/general/close_tray_on_start", prop->close_tray_on_start);
-  
+
+    gnome_config_set_int("/gtcd/keybindings/quit", prop->quit.key);
+    gnome_config_set_int("/gtcd/keybindings/play", prop->play.key);
+    gnome_config_set_int("/gtcd/keybindings/stop", prop->stop.key);
+    gnome_config_set_int("/gtcd/keybindings/tracked", prop->tracked.key);
+    gnome_config_set_int("/gtcd/keybindings/eject", prop->eject.key);
+    gnome_config_set_int("/gtcd/keybindings/back", prop->back.key);
+    gnome_config_set_int("/gtcd/keybindings/forward", prop->forward.key);
+
+    gnome_config_set_string("/cddbslave/server/address", prop->cddb_server);
+    gnome_config_set_int("/cddbslave/server/port=8880", prop->cddb_port);
+        
     gnome_config_sync();
 }
-
+        
 void changed_cb(GtkWidget *widget, void *data)
 {
     gnome_property_box_changed(GNOME_PROPERTY_BOX(pref_window));
 }
 
-void color_set_cb(GnomeColorPicker *cp, int pr, int pg, int pb)
+static void color_set_cb(GnomeColorPicker *cp, guint pr, guint pg, guint pb)
 {
-    gnome_color_picker_get_i8(cp, 
-			      &prefs.trackcolor_r, 
-			      &prefs.trackcolor_g, 
-			      &prefs.trackcolor_b,
-			      NULL);
+    prefs.trackcolor_r = pr / 256;
+    prefs.trackcolor_g = pg / 256;
+    prefs.trackcolor_b = pb / 256; 
+
     changed_cb(NULL, NULL);
 }
 
-void start_toggle_cb(GtkWidget *widget, gpointer data)
+static void start_toggle_cb(GtkWidget *widget, gpointer data)
 {
     prefs.start_action = GPOINTER_TO_INT(data);
     changed_cb(NULL, NULL);
 }
 
-void check_changed_cb(GtkWidget *widget, gboolean *data)
+static void check_changed_cb(GtkWidget *widget, gboolean *data)
 {
     if( *data )
         *data = FALSE;
@@ -118,7 +153,7 @@ void check_changed_cb(GtkWidget *widget, gboolean *data)
     changed_cb(NULL, NULL);
 }
 
-GtkWidget *create_start_frame()
+static GtkWidget *create_start_frame()
 {
     GtkWidget *start_playing;
     GtkWidget *stop_playing;
@@ -167,13 +202,13 @@ GtkWidget *create_start_frame()
     return vbox;
 }
 
-void exit_toggle_cb(GtkWidget *widget, gpointer data)
+static void exit_toggle_cb(GtkWidget *widget, gpointer data)
 {
     prefs.exit_action = GPOINTER_TO_INT(data);
     changed_cb(NULL, NULL);
 }
 
-GtkWidget *create_exit_frame()
+static GtkWidget *create_exit_frame()
 {
     GtkWidget *stop_playing;
     GtkWidget *open_tray;
@@ -223,13 +258,13 @@ GtkWidget *create_exit_frame()
     return vbox;
 }	
 
-void dev_entry_changed_cb(GtkWidget *widget, gpointer data)
+static void dev_entry_changed_cb(GtkWidget *widget, gpointer data)
 {
     prefs.cddev = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
     changed_cb(NULL, NULL);
 }
 
-void font_ok_clicked_cb(GtkWidget *widget, GtkWidget *fs)
+static void font_ok_clicked_cb(GtkWidget *widget, GtkWidget *fs)
 {
         prefs.trackfont = g_strdup(gtk_font_selection_dialog_get_font_name(
                 GTK_FONT_SELECTION_DIALOG(fs)));
@@ -237,12 +272,12 @@ void font_ok_clicked_cb(GtkWidget *widget, GtkWidget *fs)
 	changed_cb(NULL, NULL);
 }
 
-void font_cancel_clicked_cb(GtkWidget *widget, GtkWidget *fs)
+static void font_cancel_clicked_cb(GtkWidget *widget, GtkWidget *fs)
 {
         gtk_widget_destroy(fs);
 }       
         
-void font_button_cb(GtkWidget *widget, gpointer *data)
+static void font_button_cb(GtkWidget *widget, gpointer *data)
 {
         GtkWidget *fs;
         
@@ -265,10 +300,10 @@ GtkWidget *create_general_frame()
     GtkWidget *left_box, *right_box, *hbox, *vbox;
     GtkWidget *handles, *tooltips;
     
-    left_box = gtk_vbox_new(FALSE, 2);
-    right_box = gtk_vbox_new(FALSE, 2);
-    hbox = gtk_hbox_new(FALSE, 2);
-    vbox = gtk_vbox_new(FALSE, 2);
+    left_box = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
+    right_box = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
+    hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
+    vbox = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
     
     gtk_box_pack_start_defaults(GTK_BOX(hbox), left_box);
     gtk_box_pack_start_defaults(GTK_BOX(hbox), right_box);
@@ -321,7 +356,7 @@ GtkWidget *create_general_frame()
     return vbox;
 }
 
-GtkWidget *create_page()
+static GtkWidget *create_page()
 {
     GtkWidget *table;
     GtkWidget *start_frame;
@@ -332,16 +367,20 @@ GtkWidget *create_page()
     
     /* start frame */
     start_frame = gtk_frame_new(_("On Startup"));
+    gtk_container_border_width(GTK_CONTAINER(start_frame), GNOME_PAD_SMALL);
     gtk_container_add(GTK_CONTAINER(start_frame), create_start_frame());
     gtk_table_attach_defaults(GTK_TABLE(table), start_frame, 0, 1, 0, 1);
     
     /* exit frame */
     exit_frame = gtk_frame_new(_("On Exit"));
+    gtk_container_border_width(GTK_CONTAINER(exit_frame), GNOME_PAD_SMALL);
     gtk_container_add(GTK_CONTAINER(exit_frame), create_exit_frame());
     gtk_table_attach_defaults(GTK_TABLE(table), exit_frame, 0, 1, 1, 2);
     
     /* general frame */
-    general_frame = gtk_frame_new(_("General"));
+    general_frame = gtk_frame_new(NULL);
+    gtk_frame_set_shadow_type(GTK_FRAME(general_frame), GTK_SHADOW_NONE);
+    gtk_container_border_width(GTK_CONTAINER(general_frame), GNOME_PAD_SMALL);
     gtk_container_add(GTK_CONTAINER(general_frame), create_general_frame());
     gtk_table_attach_defaults(GTK_TABLE(table), general_frame, 1, 2, 0, 2);
     
@@ -352,54 +391,149 @@ GtkWidget *create_page()
 static void fill_list(KeyBinding *kb, GtkWidget *clist)
 {
     char *tmp[2];
-    
+
     tmp[0] = g_malloc(64);	/* key */
     tmp[1] = g_malloc(256);	/* desc */
 
-    g_snprintf(tmp[0], 63, "\'%c\'", kb->key);
+    g_snprintf(tmp[0], 63, "%c", toupper(kb->key->key));
     g_snprintf(tmp[1], 255, "%s", kb->desc);
     
     gtk_clist_append(GTK_CLIST(clist), tmp);
+    gtk_object_set_data(GTK_OBJECT(clist), tmp[0], kb);
 
     g_free(tmp[0]);
     g_free(tmp[1]);
 }	
 
-GtkWidget *key_page(void)
+static int entry_changed(GtkWidget *widget, GdkEvent *ev, KeyBinding *kb)
 {
-    GtkWidget *list, *frame, *box;
+    char tmp[64];
+    if(ev->type != GDK_KEY_PRESS)
+	return 0;
+    g_snprintf(tmp, 63, "%c", toupper(kb->key->key));
+    gtk_object_remove_data(GTK_OBJECT(kb->data), tmp);
+    kb->key->key = *((GdkEventKey*)ev)->string;
+
+    g_snprintf(tmp, 63, "%c", toupper(kb->key->key));
+    gtk_object_set_data(GTK_OBJECT(kb->data), tmp, kb);
+    gtk_clist_set_text(GTK_CLIST(kb->data), kb->data2, 0, tmp);
+    changed_cb(NULL, NULL);
+
+    return 1;
+}
+
+static void select_row_cb(GtkCList *clist,
+		    gint row,
+                    gint column,
+                    GdkEventButton *event,
+                    gpointer data)
+{
+    static int entry_cb=0, alt_cb=0, shift_cb=0, ctrl_cb=0;
+    GtkWidget *entry, *alt_check, *shift_check, *ctrl_check;
+    KeyBinding *kb;
+    gchar *text;
+
+    gtk_clist_get_text(clist, row, 0, &text);
+    kb = gtk_object_get_data(GTK_OBJECT(clist), text);
+    entry = gtk_object_get_data(GTK_OBJECT(clist), "entry");
+    if(!kb) /* FIXME this is broken...? */
+	return;
+    ctrl_check = gtk_object_get_data(GTK_OBJECT(clist), "ctrl_check");
+    alt_check = gtk_object_get_data(GTK_OBJECT(clist), "alt_check");
+    shift_check = gtk_object_get_data(GTK_OBJECT(clist), "shift_check");
+
+    if(entry_cb > 0) gtk_signal_disconnect(GTK_OBJECT(entry), entry_cb);
+    if(ctrl_cb > 0) gtk_signal_disconnect(GTK_OBJECT(ctrl_check), ctrl_cb);
+    if(alt_cb > 0) gtk_signal_disconnect(GTK_OBJECT(alt_check), alt_cb);
+    if(shift_cb > 0) gtk_signal_disconnect(GTK_OBJECT(shift_check), shift_cb);
+
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ctrl_check), kb->key->ctrl);
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(alt_check), kb->key->alt);
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(shift_check), kb->key->shift);
+    
+    kb->data = (GtkWidget*)clist;
+    kb->data2 = row;
+    entry_cb = gtk_signal_connect(GTK_OBJECT(entry), "event",
+				  GTK_SIGNAL_FUNC(entry_changed), kb);
+    ctrl_cb = gtk_signal_connect(GTK_OBJECT(ctrl_check), "clicked",
+		       GTK_SIGNAL_FUNC(check_changed_cb), &kb->key->ctrl);
+    alt_cb = gtk_signal_connect(GTK_OBJECT(alt_check), "clicked",
+		       GTK_SIGNAL_FUNC(check_changed_cb), &kb->key->alt);
+    shift_cb = gtk_signal_connect(GTK_OBJECT(shift_check), "clicked",
+		       GTK_SIGNAL_FUNC(check_changed_cb), &kb->key->shift);
+}
+
+static GtkWidget *key_page(void)
+{
+    GtkWidget *clist, *frame, *box, *scrolled, *mod_box;
+    GtkWidget *alt_check, *ctrl_check, *shift_check;
+    GtkWidget *label, *entry;
+
+    /* Scrolled Window */
+    scrolled = gtk_scrolled_window_new(NULL, NULL);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
+				   GTK_POLICY_AUTOMATIC,
+				   GTK_POLICY_AUTOMATIC);
 
     /* List */
-    list = gtk_clist_new(2);
-    gtk_clist_set_column_width(GTK_CLIST(list), 0, 20);
-    gtk_clist_set_policy(GTK_CLIST(list), GTK_POLICY_AUTOMATIC,
-			 GTK_POLICY_AUTOMATIC);
-    gtk_clist_set_selection_mode(GTK_CLIST(list),
+    clist = gtk_clist_new(2);
+    gtk_clist_set_column_width(GTK_CLIST(clist), 0, 30);
+    gtk_clist_set_selection_mode(GTK_CLIST(clist),
 				 GTK_SELECTION_BROWSE);
-    gtk_clist_set_column_title(GTK_CLIST(list), 0, _("Key"));
-    gtk_clist_set_column_title(GTK_CLIST(list), 1, _("Action"));
-    gtk_clist_column_titles_show(GTK_CLIST(list));
-
-    gtk_clist_column_titles_passive(GTK_CLIST(list));
-
-    g_list_foreach(keys, (GFunc)fill_list, list);
+    gtk_clist_set_column_title(GTK_CLIST(clist), 0, _("Key"));
+    gtk_clist_set_column_title(GTK_CLIST(clist), 1, _("Action"));
+    gtk_clist_column_titles_show(GTK_CLIST(clist));
+    gtk_clist_column_titles_passive(GTK_CLIST(clist));
+    gtk_clist_set_shadow_type(GTK_CLIST(clist), GTK_SHADOW_NONE);
+        
+    g_list_foreach(keys, (GFunc)fill_list, clist);
 
+    gtk_container_add(GTK_CONTAINER(scrolled), clist);
+    gtk_signal_connect(GTK_OBJECT(clist), "select_row",
+		       GTK_SIGNAL_FUNC(select_row_cb), NULL);
     /* Box */
-    box = gtk_hbox_new(FALSE, 2);
-    gtk_box_pack_start_defaults(GTK_BOX(box), list);
+    box = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
+    gtk_box_pack_start(GTK_BOX(box), scrolled, TRUE, TRUE, 0);
+
+    /* key box */
+    mod_box = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
+
+    entry = gtk_entry_new_with_max_length(1);
+    label = gtk_label_new("Click here to change:");
+
+    ctrl_check = gtk_check_button_new_with_label("Control");
+    alt_check = gtk_check_button_new_with_label("Alt");
+    shift_check = gtk_check_button_new_with_label("Shift");
+
+    gtk_box_pack_start(GTK_BOX(mod_box), label, FALSE, FALSE, GNOME_PAD);
+    gtk_box_pack_start(GTK_BOX(mod_box), entry, TRUE, TRUE, GNOME_PAD);
+#if 0				/* Not yet implemented */
+    gtk_box_pack_start_defaults(GTK_BOX(mod_box), ctrl_check);
+    gtk_box_pack_start_defaults(GTK_BOX(mod_box), alt_check);
+    gtk_box_pack_start_defaults(GTK_BOX(mod_box), shift_check);
+#endif
+
+    gtk_box_pack_start(GTK_BOX(box), mod_box, FALSE, FALSE, 0);
+
+    gtk_object_set_data(GTK_OBJECT(clist), "entry", entry);
+    gtk_object_set_data(GTK_OBJECT(clist), "ctrl_check", ctrl_check);
+    gtk_object_set_data(GTK_OBJECT(clist), "alt_check", alt_check);
+    gtk_object_set_data(GTK_OBJECT(clist), "shift_check", shift_check);
 
     /* Frame */
-    frame = gtk_frame_new(_("Keybindings (not editable YET)"));
+    frame = gtk_frame_new(_("Keybindings"));
     gtk_container_add(GTK_CONTAINER(frame), box);
+    gtk_container_border_width(GTK_CONTAINER(frame), GNOME_PAD_SMALL);
 
     gtk_widget_show_all(frame);
     return frame;
 }    
 
-void apply_cb( GtkWidget *widget, void *data )
+static void apply_cb(GtkWidget *widget, void *data)
 {       
 /* Do stuff here if needed */
-    if( prefs.tooltip )
+ 
+    if(prefs.tooltip)
         gtk_tooltips_enable(tooltips);
     else
         gtk_tooltips_disable(tooltips);
@@ -408,10 +542,18 @@ void apply_cb( GtkWidget *widget, void *data )
     save_prefs(&prefs);
 }
 
+static void cancel_cb(void)
+{
+    pref_window = NULL;
+}
+
 void preferences(GtkWidget *widget, void *data)
 {
     GtkWidget *label;
-    
+
+    if(pref_window)
+	return;
+
     pref_window = gnome_property_box_new();
     gtk_widget_realize(pref_window);
 
@@ -419,13 +561,20 @@ void preferences(GtkWidget *widget, void *data)
     gtk_notebook_append_page(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(pref_window)->notebook),
 			     create_page(), label);
     
-    
     label = gtk_label_new(_("Keybindings"));
     gtk_notebook_append_page(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(pref_window)->notebook),
     			     key_page(), label);
+
+    label = gtk_label_new(_("CDDB Settings"));
+    gtk_notebook_append_page(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(pref_window)->notebook),
+    			     create_cddb_page(), label);
 	
-    gtk_signal_connect(GTK_OBJECT(pref_window),
-		       "apply", GTK_SIGNAL_FUNC(apply_cb), NULL);
+    gtk_signal_connect(GTK_OBJECT(pref_window), "apply",
+		       GTK_SIGNAL_FUNC(apply_cb), NULL);
+    gtk_signal_connect(GTK_OBJECT(pref_window), "destroy",
+		       GTK_SIGNAL_FUNC(cancel_cb), NULL);
+    gtk_signal_connect(GTK_OBJECT(pref_window), "delete_event",
+		      GTK_SIGNAL_FUNC(cancel_cb), NULL);
 
     gtk_widget_show_all(pref_window);	
 }    
